@@ -58,6 +58,10 @@ public class PDFParser extends BaseParser
 
     private static final String PDF_HEADER = "%PDF-";
     private static final String FDF_HEADER = "%FDF-";
+    
+    private static final String PDF_DEFAULT_VERSION = "1.4";
+    private static final String FDF_DEFAULT_VERSION = "1.0";
+    
     /**
      * A list of duplicate objects found when Parsing the PDF
      * File.
@@ -183,9 +187,11 @@ public class PDFParser extends BaseParser
                 {
                     break;
                 }
+                                
                 try
                 {
-                    wasLastParsedObjectEOF = parseObject();
+                    // don't reset flag to false if it is already true
+                    wasLastParsedObjectEOF |= parseObject();
                 }
                 catch(IOException e)
                 {
@@ -204,7 +210,19 @@ public class PDFParser extends BaseParser
                          * we skipped over an object
                          */
                         LOG.warn("Parsing Error, Skipping Object", e);
+                        
+                        skipSpaces();
+                        long lastOffset = pdfSource.getOffset();
                         skipToNextObj();
+                        
+                        /* the nextObject is the one we want to skip 
+                         * so read the 'Object Number' without interpret it
+                         * in order to force the skipObject
+                         */
+                        if (lastOffset == pdfSource.getOffset()) {
+                            readStringNumber();
+                            skipToNextObj();
+                        }
                     }
                     else
                     {
@@ -338,20 +356,39 @@ public class PDFParser extends BaseParser
          */
         if (header.startsWith(PDF_HEADER))
         {
-            if(!header.matches(PDF_HEADER + "\\d.\\d"))
+            if (!header.matches(PDF_HEADER + "\\d.\\d"))
             {
-                String headerGarbage = header.substring(PDF_HEADER.length()+3, header.length()) + "\n";
-                header = header.substring(0, PDF_HEADER.length()+3);
-                pdfSource.unread(headerGarbage.getBytes("ISO-8859-1"));
+
+                if (header.length() < PDF_HEADER.length() + 3)
+                {
+                    // No version number at all, set to 1.4 as default
+                    header = PDF_HEADER + PDF_DEFAULT_VERSION;
+                    LOG.debug("No pdf version found, set to " + PDF_DEFAULT_VERSION + " as default.");
+                }
+                else
+                {
+                    String headerGarbage = header.substring(PDF_HEADER.length() + 3, header.length()) + "\n";
+                    header = header.substring(0, PDF_HEADER.length() + 3);
+                    pdfSource.unread(headerGarbage.getBytes("ISO-8859-1"));
+                }
             }
         }
         else
         {
-            if(!header.matches(FDF_HEADER + "\\d.\\d"))
+            if (!header.matches(FDF_HEADER + "\\d.\\d"))
             {
-                String headerGarbage = header.substring(FDF_HEADER.length()+3, header.length()) + "\n";
-                header = header.substring(0, FDF_HEADER.length()+3);
-                pdfSource.unread(headerGarbage.getBytes("ISO-8859-1"));
+                if (header.length() < FDF_HEADER.length() + 3)
+                {
+                    // No version number at all, set to 1.0 as default
+                    header = FDF_HEADER + FDF_DEFAULT_VERSION;
+                    LOG.debug("No fdf version found, set to " + FDF_DEFAULT_VERSION + " as default.");
+                }
+                else
+                {
+                    String headerGarbage = header.substring(FDF_HEADER.length() + 3, header.length()) + "\n";
+                    header = header.substring(0, FDF_HEADER.length() + 3);
+                    pdfSource.unread(headerGarbage.getBytes("ISO-8859-1"));
+                }
             }
         }
         document.setHeaderString(header);
@@ -507,7 +544,7 @@ public class PDFParser extends BaseParser
         //we are going to parse an normal object
         else
         {
-            int number = -1;
+            long number = -1;
             int genNum = -1;
             String objectKey = null;
             boolean missingObjectNumber = false;
@@ -520,7 +557,7 @@ public class PDFParser extends BaseParser
                 }
                 else
                 {
-                    number = readInt();
+                    number = readObjectNumber();
                 }
             }
             catch( IOException e )
@@ -529,12 +566,12 @@ public class PDFParser extends BaseParser
                 //statements after an object, of course this is nonsense
                 //but because we want to support as many PDFs as possible
                 //we will simply try again
-                number = readInt();
+                number = readObjectNumber();
             }
             if( !missingObjectNumber )
             {
                 skipSpaces();
-                genNum = readInt();
+                genNum = readGenerationNumber();
 
                 objectKey = readString( 3 );
                 //System.out.println( "parseObject() num=" + number +
@@ -676,7 +713,7 @@ public class PDFParser extends BaseParser
         /* This integer is the byte offset of the first object referenced by the xref or xref stream
          * Needed for the incremental update (PREV)
          */
-        getDocument().setStartXref(readInt());
+        getDocument().setStartXref(readLong());
         return true;
     }
 
@@ -709,8 +746,8 @@ public class PDFParser extends BaseParser
          */
         while(true)
         {
-            int currObjID = readInt(); // first obj id
-            int count = readInt(); // the number of objects in the xref table
+            long currObjID = readObjectNumber(); // first obj id
+            long count = readLong(); // the number of objects in the xref table
             skipSpaces();
             for(int i = 0; i < count; i++)
             {
