@@ -73,6 +73,12 @@ public class PageDrawer extends PDFStreamEngine
     private static final Log LOG = LogFactory.getLog(PageDrawer.class);
 
     private Graphics2D graphics;
+    
+    /**
+     * clipping winding rule used for the clipping path.
+     */
+    private int clippingWindingRule = -1;
+
     /**
      * Size of the page.
      */
@@ -118,7 +124,7 @@ public class PageDrawer extends PDFStreamEngine
             PDResources resources = page.findResources();
             processStream( page, resources, page.getContents().getStream() );
         }
-        List annotations = page.getAnnotations();
+        List<PDAnnotation> annotations = page.getAnnotations();
         for( int i=0; i<annotations.size(); i++ )
         {
             PDAnnotation annot = (PDAnnotation)annotations.get( i );
@@ -131,16 +137,24 @@ public class PageDrawer extends PDFStreamEngine
                 {
                     appearanceName = "default";
                 }
-                Map appearanceMap = appearDictionary.getNormalAppearance();
+                Map<String, PDAppearanceStream> appearanceMap = appearDictionary.getNormalAppearance();
                 if (appearanceMap != null) 
                 { 
                     PDAppearanceStream appearance = 
                         (PDAppearanceStream)appearanceMap.get( appearanceName ); 
                     if( appearance != null ) 
                     { 
-                        g.translate( (int)rect.getLowerLeftX(), (int)-rect.getLowerLeftY() ); 
+                        Point2D point = new Point2D.Float(rect.getLowerLeftX(), rect.getLowerLeftY());
+                        Matrix matrix = appearance.getMatrix();
+                        if (matrix != null) 
+                        {
+                            // transform the rectangle using the given matrix 
+                            AffineTransform at = matrix.createAffineTransform();
+                            at.transform(point, point);
+                        }
+                        g.translate( (int)point.getX(), -(int)point.getY() );
                         processSubStream( page, appearance.getResources(), appearance.getStream() ); 
-                        g.translate( (int)-rect.getLowerLeftX(), (int)+rect.getLowerLeftY() ); 
+                        g.translate( -(int)point.getX(), (int)point.getY() ); 
                     }
                 }
             }
@@ -148,6 +162,17 @@ public class PageDrawer extends PDFStreamEngine
 
     }
 
+    /**
+     * Remove all cached resources.
+     */
+    public void dispose()
+    {
+        graphics = null;
+        linePath = null;
+        page = null;
+        pageSize = null;
+    }
+
     /**
      * You should override this method if you want to perform an action when a
      * text is being processed.
@@ -406,27 +431,52 @@ public class PageDrawer extends PDFStreamEngine
      *
      * @param windingRule The winding rule this path will use.
      * 
+     * @deprecated use {@link #setClippingWindingRule(int)} instead
+     * 
      */
     public void setClippingPath(int windingRule)
     {
-        PDGraphicsState graphicsState = getGraphicsState();
-        GeneralPath clippingPath = (GeneralPath)getLinePath().clone();
-        clippingPath.setWindingRule(windingRule);
-        // If there is already set a clipping path, we have to intersect the new with the existing one
-        if (graphicsState.getCurrentClippingPath() != null) 
+        setClippingWindingRule(windingRule);
+    }
+    
+    /**
+     * Set the clipping winding rule.
+     *
+     * @param windingRule The winding rule which will be used for clipping.
+     * 
+     */
+    public void setClippingWindingRule(int windingRule)
+    {
+        clippingWindingRule = windingRule;
+    }
+
+    /**
+     * Set the clipping Path.
+     *
+     */
+    public void endPath()
+    {
+        if (clippingWindingRule > -1)
         {
-            Area currentArea = new Area(getGraphicsState().getCurrentClippingPath());
-            Area newArea = new Area(clippingPath);
-            currentArea.intersect(newArea);
-            graphicsState.setCurrentClippingPath(currentArea);
-        }
-        else 
-        {
-            graphicsState.setCurrentClippingPath(clippingPath);
+            PDGraphicsState graphicsState = getGraphicsState();
+            GeneralPath clippingPath = (GeneralPath)getLinePath().clone();
+            clippingPath.setWindingRule(clippingWindingRule);
+            // If there is already set a clipping path, we have to intersect the new with the existing one
+            if (graphicsState.getCurrentClippingPath() != null) 
+            {
+                Area currentArea = new Area(getGraphicsState().getCurrentClippingPath());
+                Area newArea = new Area(clippingPath);
+                currentArea.intersect(newArea);
+                graphicsState.setCurrentClippingPath(currentArea);
+            }
+            else 
+            {
+                graphicsState.setCurrentClippingPath(clippingPath);
+            }
+            clippingWindingRule = -1;
         }
         getLinePath().reset();
     }
-    
     /**
      * Draw the AWT image. Called by Invoke.
      * Moved into PageDrawer so that Invoke doesn't have to reach in here for Graphics as that breaks extensibility.
