@@ -116,6 +116,14 @@ public class PDDocument implements Pageable
      * from this documents.
      */
     private boolean allSecurityToBeRemoved = false;
+
+    /**
+     * Keep tracking customized documentId for the trailer. If null, a new 
+     * id will be generated for the document. This ID doesn't represent the
+     * actual documentId from the trailer.
+     */
+    private Long documentId;
+
     
     /**
      * Constructor, creates a new PDF Document with no pages.  You need to add
@@ -366,6 +374,11 @@ public class PDDocument implements Pageable
 
         List<PDField> fields = acroForm.getFields();
         PDSignatureField signatureField = null;
+        if(fields == null) 
+        {
+            fields = new ArrayList();
+            acroForm.setFields(fields);
+        }
         for ( PDField pdField : fields )
         {
             if (pdField instanceof PDSignatureField)
@@ -1627,5 +1640,15 @@ public class PDDocument implements Pageable
     {
         allSecurityToBeRemoved = removeAllSecurity;
     }
+    
+    public Long getDocumentId() 
+    {
+      return documentId;
+    }
+    
+    public void setDocumentId(Long docId)
+    {
+      documentId = docId;
+    }
 }
 
